html, body, #wrap {
	color: #333;
	background: #f7f7f7;
	font-family: sans-serif;
}
#wrap {
	max-width: 40em;
	margin: 2em auto;
	padding: 0 1em 3em 1em;
	line-height: 1.3em;
}
h1 {
	border-bottom: 1px solid #ccc;
	margin: 2em 0 .4em 0;
	padding: 0 0 .2em 0;
	font-weight: normal;
}
li {
	margin: 1em 0;
}
a {
	color: #047;
	background: #fff;
	text-decoration: none;
	border-bottom: 1px solid #aaa;
	border-radius: .2em;
	padding: .2em .8em;
}
table {
	border-collapse: collapse;
}
.vols td,
.vols th {
	padding: .3em .6em;
	text-align: left;
}
.num {
	border-right: 1px solid #bbb;
}
.num td {
	padding: .1em .7em .1em 0;
}
.num td:first-child {
	text-align: right;
}
.btns {
	margin: 1em 0;
}


html.dark,
html.dark body,
html.dark #wrap {
	background: #222;
	color: #ccc;
}
html.dark h1 {
	border-color: #777;
}
html.dark a {
	color: #fff;
	background: #057;
	border-color: #37a;
}
html.dark input {
	color: #fff;
	background: #624;
	border: 1px solid #c27;
	border-width: 1px 0 0 0;
	border-radius: .5em;
	padding: .5em .7em;
	margin: 0 .5em 0 0;
}
html.dark .num {
	border-color: #777;
}