
#op_up2k {
	padding: 0 1em 1em 1em;
}
#u2form {
	position: absolute;
	top: 0;
	left: 0;
	width: 2px;
	height: 2px;
	overflow: hidden;
}
#u2form input {
	background: #444;
	border: 0px solid #444;
	outline: none;
}
#u2err.err {
	color: #f87;
	padding: .5em;
}
#u2err.msg {
	color: #999;
	padding: .5em;
	font-size: .9em;
}
#u2btn {
	color: #eee;
	background: #555;
	background: -moz-linear-gradient(top, #367 0%, #489 50%, #38788a 51%, #367 100%);
	background: -webkit-linear-gradient(top, #367 0%, #489 50%, #38788a 51%, #367 100%);
	background: linear-gradient(to bottom, #367 0%, #489 50%, #38788a 51%, #367 100%);
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#489', endColorstr='#38788a', GradientType=0);
	text-decoration: none;
	line-height: 1.3em;
	border: 1px solid #222;
	border-radius: .4em;
	text-align: center;
	font-size: 1.5em;
	margin: .5em auto;
	padding: .8em 0;
	width: 16em;
	cursor: pointer;
	box-shadow: .4em .4em 0 #111;
}
#op_up2k.srch #u2btn {
	background: linear-gradient(to bottom, #ca3 0%, #fd8 50%, #fc6 51%, #b92 100%);
	text-shadow: 1px 1px 1px #fc6;
	color: #333;
}
#u2conf #u2btn {
	margin: -1.5em 0;
	padding: .8em 0;
	width: 100%;
	max-width: 12em;
	display: inline-block;
}
#u2conf #u2btn_cw {
	text-align: right;
}
#u2notbtn {
	display: none;
	text-align: center;
	background: #333;
	padding-top: 1em;
}
#u2notbtn * {
	line-height: 1.3em;
}
#u2tab {
	margin: 3em auto;
	width: calc(100% - 2em);
	max-width: 100em;
}
#op_up2k.srch #u2tab {
	max-width: none;
}
#u2tab td {
	border: 1px solid #ccc;
	border-width: 0 0px 1px 0;
	padding: .1em .3em;
}
#u2tab td:nth-child(2) {
	width: 5em;
	white-space: nowrap;
}
#u2tab td:nth-child(3) {
	width: 40%;
}
#op_up2k.srch #u2tab td:nth-child(3) {
	font-family: sans-serif;
	width: auto;
}
#u2tab tbody tr:hover td {
	background: #222;
}
#u2cards {
	padding: 1em 0 .3em 1em;
	margin: 1.5em auto -2.5em auto;
	white-space: nowrap;
	text-align: center;
	overflow: hidden;
}
#u2cards.w {
	width: 45em;
	text-align: left;
}
#u2cards a {
	padding: .2em 1em;
	border: 1px solid #777;
	border-width: 0 0 1px 0;
	background: linear-gradient(to bottom, #333, #222);
}
#u2cards a:first-child {
	border-radius: .4em 0 0 0;
}
#u2cards a:last-child {
	border-radius: 0 .4em 0 0;
}
#u2cards a.act {
	padding-bottom: .5em;
	border-width: 1px 1px .1em 1px;
	border-radius: .3em .3em 0 0;
	margin-left: -1px;
	background: linear-gradient(to bottom, #464, #333 80%);
	box-shadow: 0 -.17em .67em #280;
	border-color: #7c5 #583 #333 #583;
	position: relative;
	color: #fd7;
}
#u2cards span {
	color: #fff;
}
#u2conf {
	margin: 1em auto;
	width: 30em;
}
#u2conf.has_btn {
	width: 48em;
}
#u2conf * {
	text-align: center;
	line-height: 1em;
	margin: 0;
	padding: 0;
	border: none;
	outline: none;
}
#u2conf .txtbox {
	width: 3em;
	color: #fff;
	background: #444;
	border: 1px solid #777;
	font-size: 1.2em;
	padding: .15em 0;
	height: 1.05em;
}
#u2conf .txtbox.err {
	background: #922;
}
#u2conf a {
	color: #fff;
	background: #c38;
	text-decoration: none;
	border-radius: .1em;
	font-size: 1.5em;
	padding: .1em 0;
	margin: 0 -1px;
	width: 1.5em;
	height: 1em;
	display: inline-block;
	position: relative;
	bottom: -0.08em;
}
#u2conf input+a {
	background: #d80;
}
#u2conf label {
	font-size: 1.6em;
	width: 2em;
	height: 1em;
	padding: .4em 0;
	display: block;
	border-radius: .25em;
}
#u2conf input[type="checkbox"] {
	position: relative;
	opacity: .02;
	top: 2em;
}
#u2conf input[type="checkbox"]+label {
	position: relative;
	background: #603;
	border-bottom: .2em solid #a16;
	box-shadow: 0 .1em .3em #a00 inset;
}
#u2conf input[type="checkbox"]:checked+label {
	background: #6a1;
	border-bottom: .2em solid #efa;
	box-shadow: 0 .1em .5em #0c0;
}
#u2conf input[type="checkbox"]+label:hover {
	box-shadow: 0 .1em .3em #fb0;
	border-color: #fb0;
}
#op_up2k.srch #u2conf td:nth-child(1)>*,
#op_up2k.srch #u2conf td:nth-child(2)>*,
#op_up2k.srch #u2conf td:nth-child(3)>* {
	background: #777;
	border-color: #ccc;
	box-shadow: none;
	opacity: .2;
}
#u2cdesc {
	position: absolute;
	width: 34em;
	left: calc(50% - 15em);
	background: #222;
	border: 0 solid #555;
	text-align: center;
	overflow: hidden;
	margin: 0 -2em;
	padding: 0 1em;
	height: 0;
	opacity: .1;
	transition: all 0.14s ease-in-out;
	box-shadow: 0 .2em .5em #222;
	border-radius: .4em;
	z-index: 1;
}
#u2cdesc.show {
	padding: 1em;
	height: auto;
	border-width: .2em 0;
	opacity: 1;
}
#u2foot {
	color: #fff;
	font-style: italic;
}
#u2foot span {
	color: #999;
	font-size: .9em;
}
#u2footfoot {
	margin-bottom: -1em;
}
.prog {
	font-family: monospace;
}
#u2tab a>span {
	font-weight: bold;
	font-style: italic;
	color: #fff;
	padding-left: .2em;
}
#u2cleanup {
	float: right;
	margin-bottom: -.3em;
}





html.light #u2btn {
	box-shadow: .4em .4em 0 #ccc;
}
html.light #u2cards span {
	color: #000;
}
html.light #u2cards a {
	background: linear-gradient(to bottom, #eee, #fff);
}
html.light #u2cards a.act {
	color: #037;
	background: inherit;
	box-shadow: 0 -.17em .67em #0ad;
	border-color: #09c #05a #eee #05a;
}
html.light #u2conf .txtbox {
	background: #fff;
	color: #444;
}
html.light #u2conf .txtbox.err {
	background: #f96;
	color: #300;
}
html.light #u2cdesc {
	background: #fff;
	border: none;
}
html.light #op_up2k.srch #u2btn {
	border-color: #a80;
}
html.light #u2foot {
	color: #000;
}
html.light #u2tab tbody tr:hover td {
	background: #fff;
}
